package com.qmth.ops.api.controller.admin;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.ops.api.binder.FileFormatBinder;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.dto.CodeNameBean;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.biz.domain.*;
import com.qmth.ops.biz.service.*;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/config")
public class ConfigController {

    @Resource
    private ConfigGroupService configGroupService;

    @Resource
    private ConfigService configService;

    @Resource
    private AppService appService;

    @Resource
    private ModuleService moduleService;

    @Resource
    private VersionService versionService;

    @Resource
    private EnvService envService;

    @InitBinder
    public void initBinder(WebDataBinder dataBinder) {
        dataBinder.addCustomFormatter(new FileFormatBinder());
    }

    @RequestMapping("/groups")
    @Aac(auth = BOOL.FALSE)
    public List<ConfigGroup> listGroups() {
        return configGroupService.getConfigGroupList();
    }

    @RequestMapping("/modes")
    @Aac(auth = BOOL.FALSE)
    public Object listModes() {
        return Arrays.stream(ConfigMode.values()).map(model -> new CodeNameBean(model.getCode(), model.getName()))
                .toArray();
    }

    @PostMapping("/baseline")
    public List<ConfigItem> listBaseline(@RequestAttribute AdminSession accessEntity, @RequestParam Long appId,
            @RequestParam Long versionId, @RequestParam Long moduleId) {
        accessEntity.validateApp(appService.getById(appId));
        return configService.listBaseline(appId, versionId, moduleId);
    }

    @PostMapping("/baseline/update")
    public Object updateBaseline(@RequestAttribute AdminSession accessEntity, @RequestParam Long appId,
            @RequestParam Long versionId, @RequestParam Long moduleId, @RequestParam MultipartFile file,
            @RequestParam FileFormat extension, @RequestParam(required = false) Long inheritVersionId)
            throws IOException {
        accessEntity.validateRole(Role.DEV);
        accessEntity.validateApp(appService.getById(appId));
        return configService.updateBaseline(appService.getById(appId), versionService.getById(versionId),
                moduleService.getById(moduleId), file.getInputStream(), extension,
                inheritVersionId != null ? versionService.getById(inheritVersionId) : null);
    }

    @PostMapping("/baseline/item/update")
    public ConfigItem updateBaselineItem(@RequestAttribute AdminSession accessEntity, ConfigItem item) {
        accessEntity.validateRole(Role.DEV);
        accessEntity.validateApp(appService.getById(item.getAppId()));
        return configService.updateBaselineItem(item);
    }

    @PostMapping("/list")
    public List<ConfigItem> listConfigItem(@RequestAttribute AdminSession accessEntity, @RequestParam Long appId,
            @RequestParam Long versionId, @RequestParam Long moduleId, @RequestParam Long envId) {
        accessEntity.validateApp(appService.getById(appId));
        Env env = envService.getById(envId);
        List<ConfigItem> list = configService.listConfigItem(appId, versionId, moduleId, env.getId());
        //非环境可编辑用户，需要隐藏机密信息
        if (!accessEntity.getUser().hasRole(env.getType().getRole())) {
            for (ConfigItem item : list) {
                if (item.getKey().contains("secret") || item.getKey().contains("password")) {
                    item.setValue("***");
                }
            }
        }
        return list;
    }

    @PostMapping("/item/update")
    public ConfigItem updateConfigItem(@RequestAttribute AdminSession accessEntity, ConfigItem item) {
        accessEntity.validateRole(Role.TEST, Role.OPS);
        accessEntity.validateApp(appService.getById(item.getAppId()));
        accessEntity.validateEnv(envService.getById(item.getEnvId()).getType());
        return configService.updateConfigItem(item);
    }

}

