package com.qmth.ops.api.controller.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.dto.CodeNameBean;
import com.qmth.ops.api.dto.LoginResult;
import com.qmth.ops.api.dto.UserForm;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.biz.domain.Role;
import com.qmth.ops.biz.domain.User;
import com.qmth.ops.biz.query.UserQuery;
import com.qmth.ops.biz.service.AppUserService;
import com.qmth.ops.biz.service.UserService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.List;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/user")
public class UserController {

    @Resource
    private UserService userService;

    @Resource
    private AppUserService appUserService;

    @PostMapping("/login")
    @Aac(auth = BOOL.FALSE)
    public LoginResult login(@Validated(UserForm.LoginGroup.class) UserForm form) {
        User user = userService.findByLoginName(form.getLoginName());
        if (user == null) {
            throw new ParameterException("登录名错误");
        }
        if (!user.buildPassword(form.getPassword()).equals(user.getPassword())) {
            throw new ParameterException("密码错误");
        }
        return new AdminSession(user, appUserService).getLoginResult();
    }

    @RequestMapping("/roles")
    @Aac(auth = BOOL.FALSE)
    public Object getRoles() {
        return Arrays.stream(Role.values()).map(role -> new CodeNameBean(role.getCode(), role.getName())).toArray();
    }

    @PostMapping("/query")
    public IPage<User> query(@RequestAttribute AdminSession accessEntity, UserQuery query) {
        accessEntity.validateRole(Role.ADMIN);
        return userService.query(query);
    }

    @PostMapping("/list")
    public List<User> list(@RequestAttribute AdminSession accessEntity, UserQuery query) {
        accessEntity.validateRole(Role.ADMIN);
        return userService.list(query);
    }

    @PostMapping("/insert")
    public User insert(@RequestAttribute AdminSession accessEntity,
            @Validated(UserForm.InsertGroup.class) UserForm form) {
        accessEntity.validateRole(Role.ADMIN);
        return userService.insert(form.build());
    }

    @PostMapping("/update")
    public User update(@RequestAttribute AdminSession accessEntity,
            @Validated(UserForm.UpdateGroup.class) UserForm form) {
        accessEntity.validateRole(Role.ADMIN);
        return userService.update(form.build());
    }

    @PostMapping("/change_password")
    public User changePassword(@RequestAttribute AdminSession accessEntity, @RequestParam String password) {
        User user = accessEntity.getUser();
        user.setPassword(password);
        return userService.changePassword(user);
    }

}
