package com.qmth.ops.api.controller.admin;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.ops.api.binder.VersionNumberBinder;
import com.qmth.ops.api.constants.OpsApiConstants;
import com.qmth.ops.api.security.AdminSession;
import com.qmth.ops.biz.domain.Role;
import com.qmth.ops.biz.domain.Version;
import com.qmth.ops.biz.query.VersionQuery;
import com.qmth.ops.biz.service.AppService;
import com.qmth.ops.biz.service.VersionService;
import com.qmth.ops.biz.utils.VersionNumber;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

@RestController
@RequestMapping(OpsApiConstants.ADMIN_URI_PREFIX + "/version")
public class VersionController {

    @Resource
    private VersionService versionService;

    @Resource
    private AppService appService;

    @InitBinder
    public void initBinder(WebDataBinder dataBinder) {
        dataBinder.addCustomFormatter(new VersionNumberBinder());
    }

    @PostMapping("/insert")
    public Version insert(@RequestAttribute AdminSession accessEntity, @RequestParam Long appId,
            @RequestParam VersionNumber name) {
        accessEntity.validateRole(Role.ADMIN, Role.DEV);
        accessEntity.validateApp(appService.getById(appId));
        return versionService.insert(appService.getById(appId), name);
    }

    @PostMapping("/update")
    public Version list(@RequestAttribute AdminSession accessEntity, Version version) {
        accessEntity.validateRole(Role.ADMIN, Role.DEV);
        accessEntity.validateApp(appService.getById(versionService.getById(version.getId()).getAppId()));
        return versionService.update(version);
    }

    @PostMapping("/query")
    public IPage<Version> query(VersionQuery query) {
        return versionService.query(query);
    }

}
