package com.qmth.ops.api.security;

import com.qmth.boot.core.exception.UnauthorizedException;
import com.qmth.boot.core.security.model.AccessEntity;
import com.qmth.ops.api.dto.LoginResult;
import com.qmth.ops.biz.domain.App;
import com.qmth.ops.biz.domain.EnvType;
import com.qmth.ops.biz.domain.Role;
import com.qmth.ops.biz.domain.User;
import com.qmth.ops.biz.service.AppUserService;

public class AdminSession implements AccessEntity {

    private User user;

    private AppUserService appUserService;

    public AdminSession(User user, AppUserService appUserService) {
        this.user = user;
        this.appUserService = appUserService;
    }

    public LoginResult getLoginResult() {
        LoginResult result = new LoginResult();
        result.setRole(user.getRole());
        result.setSession(getIdentity());
        result.setToken(getSecret());
        return result;
    }

    public User getUser() {
        return user;
    }

    public void validateRole(Role... roles) {
        if (!user.hasRole(roles)) {
            throw new UnauthorizedException("没有操作权限");
        }
    }

    public void validateApp(App app) {
        if (!user.hasRole(Role.OPS, Role.ADMIN) && !appUserService.exist(app, user)) {
            throw new UnauthorizedException("没有应用操作权限");
        }
    }

    public void validateEnv(EnvType envType) {
        if (!user.hasRole(envType.getRole())) {
            throw new UnauthorizedException("没有环境操作权限");
        }
    }

    @Override
    public String getIdentity() {
        return user.getId().toString();
    }

    @Override
    public String getSecret() {
        return user.buildAccessToken();
    }
}
