package com.qmth.ops.biz.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.ops.biz.dao.AppDao;
import com.qmth.ops.biz.domain.App;
import com.qmth.ops.biz.domain.AppDTO;
import com.qmth.ops.biz.domain.Version;
import com.qmth.ops.biz.query.AppQuery;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.List;

@Service
public class AppService extends ServiceImpl<AppDao, App> {

    @Resource
    private AppDao appDao;

    @Transactional
    public App insert(App app) {
        app.setCreateTime(System.currentTimeMillis());
        app.setUpdateTime(app.getCreateTime());
        appDao.insert(app);
        return app;
    }

    @Transactional
    public void update(App app) {
        appDao.update(app, new LambdaUpdateWrapper<App>().set(app.getCode() != null, App::getCode, app.getCode())
                .set(app.getName() != null, App::getName, app.getName())
                .set(App::getUpdateTime, System.currentTimeMillis()).eq(App::getId, app.getId()));
    }

    public AppDTO findDTO(Long id) {
        AppQuery query = new AppQuery();
        query.setId(id);
        List<AppDTO> list = list(query);
        return list.isEmpty() ? null : list.get(0);
    }

    public IPage<AppDTO> query(AppQuery query) {
        return appDao.findByQuery(query, query);
    }

    public List<AppDTO> list(AppQuery query) {
        return appDao.findByQuery(query);
    }

    public App findByCode(String code) {
        return appDao.selectOne(new LambdaQueryWrapper<App>().eq(App::getCode, code));
    }

    @Transactional
    public void setMasterVersion(@NotNull App app, @NotNull Version version) {
        if (!version.getAppId().equals(app.getId())) {
            throw new ParameterException("指定版本不属于当前应用");
        }
        appDao.update(app, new LambdaUpdateWrapper<App>().set(App::getMasterVersionId, version.getId())
                .set(App::getUpdateTime, System.currentTimeMillis()).eq(App::getId, app.getId()));
    }
}

