package com.qmth.ops.biz.service;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.type.CollectionType;
import com.qmth.ops.biz.domain.ConfigGroup;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

@Service
public class ConfigGroupService {

    private static final Logger log = LoggerFactory.getLogger(ConfigGroupService.class);

    private static String[] FILE_PATH = { "config/qmth-boot.json", "config/spring.json" };

    private List<ConfigGroup> configGroupList = new ArrayList<>();

    public ConfigGroupService() throws IOException {
        loadGroupFile(configGroupList, FILE_PATH);
    }

    private void loadGroupFile(List<ConfigGroup> configGroupList, String... path) throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        CollectionType collectionType = mapper.getTypeFactory()
                .constructCollectionType(ArrayList.class, ConfigGroup.class);
        for (String filePath : path) {
            List<ConfigGroup> list = mapper
                    .readValue(this.getClass().getClassLoader().getResourceAsStream(filePath), collectionType);
            log.info(filePath + " size=" + list.size());
            configGroupList.addAll(list);
        }
    }

    public List<ConfigGroup> getConfigGroupList() {
        return configGroupList;
    }

    public boolean accept(String key, String value) {
        for (ConfigGroup group : configGroupList) {
            if (group.accept(key, value)) {
                return true;
            }
        }
        return false;
    }
}
