package com.qmth.ops.biz.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.ops.biz.dao.EnvDao;
import com.qmth.ops.biz.domain.Env;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

@Service
public class EnvService extends ServiceImpl<EnvDao, Env> {

    @Resource
    private EnvDao envDao;

    @Transactional
    public Env insert(Env env) {
        env.setCreateTime(System.currentTimeMillis());
        env.setUpdateTime(env.getCreateTime());
        envDao.insert(env);
        return env;
    }

    @Transactional
    public Env update(Env env) {
        envDao.update(env, new LambdaUpdateWrapper<Env>().set(env.getCode() != null, Env::getCode, env.getCode())
                .set(env.getName() != null, Env::getName, env.getName())
                .set(Env::getUpdateTime, System.currentTimeMillis()).eq(Env::getId, env.getId()));
        return envDao.selectById(env.getId());
    }

    public List<Env> list(Long appId) {
        return envDao.selectList(new LambdaQueryWrapper<Env>().eq(Env::getAppId, appId));
    }

    public Env findByAppAndCode(Long appId, String code) {
        return envDao.selectOne(new LambdaQueryWrapper<Env>().eq(Env::getAppId, appId).eq(Env::getCode, code));
    }

}

